<?php

namespace Iwaydev\LaravelLogger;

use Monolog\Formatter\JsonFormatter as MonologJsonFormatter;
use Monolog\LogRecord;

class JsonFormatter extends MonologJsonFormatter
{
    /**
     * @param int $batchMode
     * @param bool $appendNewline
     * @param bool $ignoreEmptyContextAndExtra
     * @param bool $includeStacktraces
     * @param string $serverName
     * @param string $projectName
     * @param string $logCategory
     * @param array $tracking
     */
    public function __construct(
        int $batchMode = MonologJsonFormatter::BATCH_MODE_JSON,
        bool $appendNewline = true,
        bool $ignoreEmptyContextAndExtra = false,
        bool $includeStacktraces = false,
        private string $serverName = '',
        private string $projectName = '',
        private string $logCategory = '',
        private array $tracking = [],
        private array $paramsMasking = [],
        private string $entryPoint = ''
    ) {
        MonologJsonFormatter::__construct($batchMode, $appendNewline, $ignoreEmptyContextAndExtra, $includeStacktraces);
    }

    /**
     * @param LogRecord $record
     *
     * @return string
     */
    public function format(LogRecord $record): string
    {
        $normalized = [];
        $context = $record->context;
        $request = \request();

        $currentRoute = $request->getMethod() . ' ' . $request->getPathInfo();
        $tracking = $this->getTracking($currentRoute);
        $queryParams = [];
        $bodyParams = [];
        if (isset($tracking['params']) && $tracking['params'] === true) {
            $queryParams = $request->query();
            $bodyParams = $request->post();
        }

        $headers = [];
        if (isset($tracking['headers']) && $tracking['headers'] === true) {
            $headers = $request->header();
        }

        if (isset($context['exception'])) {
            $errorCode = $context['exception']['error_code'];
            $file = $context['exception']['file'];
            $trace = $context['exception']['trace'];
            unset($context['exception']);
        }
        $message = empty($context)
            ? $record->message
            : $this->toJson([$record->message, $context]);

        $data = [
            'entry_point' => empty($this->entryPoint) ? $request->getHost() : $this->entryPoint,
            'ip' => $request->ip() ?? '-',
            'uuid' => $request->header('X-UUID', '-'),
            'level' => $record->level->toPsrLogLevel(),
            'message' => $message,
            'headers' => $this->masking($headers),
            'query_params' => $this->masking($queryParams),
            'body_params' => $this->masking($bodyParams),
            'error_code' => $errorCode ?? '-',
            'file' => $file ?? '-',
            'trace' => $trace ?? '-',
        ];

        $normalized['server'] = $this->serverName;
        $normalized['project'] = $this->projectName;
        $normalized['category'] = $this->logCategory;
        $normalized['data'] = $data;

        return $this->toJson($normalized, true) . ($this->appendNewline ? "\n" : '');
    }

    /**
     * @param array $parameters
     *
     * @return array
     */
    private function masking(array $parameters): array
    {
        foreach ($parameters as $parameterName => $value) {
            $parameters[$parameterName] = in_array($parameterName, $this->paramsMasking)
                ? '*****'
                : $value;
        }

        return $parameters;
    }

    /**
     * @param string $currentRoute
     *
     * @return array
     */
    private function getTracking(string $currentRoute): array
    {
        $tracking = $this->tracking['default'] ?? [];
        foreach ($this->tracking as $pattern => $config) {
            if (isPresentInConfig([$pattern => $config], $currentRoute)) {
                return array_merge($tracking, $config);
            }
        }

        return $tracking;
    }
}
