<?php

namespace Iwaydev\LaravelLogger\Middleware;

use Iwaydev\LaravelLogger\Traits\LoggerTrait;
use Closure;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Symfony\Component\HttpFoundation\Response;

use function Amp\Promise\timeout;

/**
 * Логируем запрос-ответ
 *
 * @package App\Http\Middleware
 */
class RequestResponseLog
{
    use LoggerTrait;

    /**
     * Handle an incoming request.
     *
     * @param Request $request
     * @param Closure $next
     *
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $start = microtime(true);

        $this->setLogger(Log::channel('request_response'), $request->header('X-UUID', ''));

        /** @var Response $response */
        $response = $next($request);

        $requestExecutionTime = microtime(true) - $start;
        if (!$this->skipLogging($request)) {
            $this->logResponse($response, $requestExecutionTime, $this->getCurrentRoute($request));
        }

        return $response;
    }

    /**
     * @param Request $request
     *
     * @return string
     */
    private function getCurrentRoute(Request $request): string
    {
        return $request->getMethod() . ' ' . $request->getPathInfo();
    }

    /**
     * @param Request $request
     *
     * @return bool
     */
    private function skipLogging(Request $request): bool
    {
        if (!$this->skipByIp($request->ip())) {
            return $this->skipByRoute($this->getCurrentRoute($request));
        }

        return true;
    }

    /**
     * @param string $currentRoute
     *
     * @return bool
     */
    private function skipByRoute(string $currentRoute): bool
    {
        $onlyRoutes = config('logging.request_response.only_routes');
        $excludeRoutes = config('logging.request_response.exclude_routes');
        if (!empty($onlyRoutes)) {
            return !isPresentInConfig($onlyRoutes, $currentRoute);
        }

        if (!empty($excludeRoutes)) {
            return isPresentInConfig($excludeRoutes, $currentRoute);
        }

        return false;
    }

    /**
     * @param string|null $currentIp
     *
     * @return bool
     */
    private function skipByIp(?string $currentIp = null): bool
    {
        if ($currentIp === null) {
            return false;
        }

        $onlyIps = config('logging.request_response.only_ips');
        $excludeIps = config('logging.request_response.exclude_ips');
        if (!empty($onlyIps)) {
            return !isPresentInConfig($onlyIps, $currentIp);
        }

        if (!empty($excludeIps)) {
            return isPresentInConfig($excludeIps, $currentIp);
        }

        return false;
    }

    /**
     * @param Response $response
     * @param int $requestExecutionTime
     * @param string $currentRoute
     *
     * @return void
     */
    private function logResponse(Response $response, int $requestExecutionTime, string $currentRoute): void
    {
        $context = [
            'action' => $currentRoute,
            'request_execution_time' => round($requestExecutionTime, 3) . ' sec',
        ];
        if ($response->isClientError() || $response->isServerError()) {
            if ($response->exception) {
                $this->logException($response->exception, $context);
            } else {
                $this->logError('Error!', array_merge($context, ['response' => $response->getContent()]));
            }
        } else {
            $this->logInfo('', array_merge($context, ['response' => $response->getContent()]));
        }
    }
}
